# generated by datamodel-codegen:
#   filename:  b.json
#   timestamp: 2024-10-29T04:06:30+00:00

from __future__ import annotations

import html
from typing import List

from pydantic import BaseModel, field_validator


class PresenceItem(BaseModel):
    model_config = {'extra': 'allow'}

    id: str
    notes: str = ''
    absent: bool = False
    hourOut: str = ''
    hourIn: str = ''

    @field_validator('hourOut', 'hourIn')
    def parse_hours(cls, value):
        if value == '--:--':
            return ''
        return value


class Child(BaseModel):
    model_config = {'extra': 'allow'}

    id: str
    name: str
    presence: List[PresenceItem]

    @field_validator('name')
    def parse_name(cls, value):
        if isinstance(value, str):
            value = html.unescape(value)
        return value


class AbsentReason(BaseModel):
    model_config = {'extra': 'allow'}

    id: str
    name: str
    selected: str


class SchoolQRCodeInfo(BaseModel):
    model_config = {'extra': 'allow'}

    translate: dict[str, str]
    child: dict[str, Child]
    absentReasons: List[AbsentReason]
    numChild: int
    schoolDate: str
    schoolhour: str
    hasPresence: bool
    hasQrenable: bool
    qrdate: str
    schoolslug: str
    schoolId: str
    presencein: str
    presenceout: str
    presencesingleurl: str
    presencesinglenoteurl: str
    absenturl: str
    presenceinurl: str
    presenceouturl: str
    multiplePresence: int
    locale: str
    language: str



# generated by datamodel-codegen:
#   filename:  a.json
#   timestamp: 2024-10-29T04:01:01+00:00

from __future__ import annotations

from pydantic import BaseModel


class Links(BaseModel):
    model_config = {'extra': 'allow'}

    info: str
    clinical: str
    inbox: str
    outbox: str
    payments: str
    reportdaily: str
    reportprogress: str
    services: str
    gallery: str
    calendar: str
    contacts: str


class Notifications(BaseModel):
    model_config = {'extra': 'allow'}

    message: str
    payment: str
    report: str
    gallery: str
    campain: str
    event: str


class Child(BaseModel):
    model_config = {'extra': 'allow'}

    name: str
    photo: str
    school: str
    links: Links
    role: str
    consult: bool
    onlypresence: bool
    notifications: Notifications
    hasDailyReport: bool
    hasProgressReport: bool


class Home(BaseModel):
    model_config = {'extra': 'allow'}

    translate: dict[str, str]
    schoolname: str
    showpayment: bool
    children: dict[str, Child]
    placardlink: str
    userlink: str
    uploadpic: str
    rgpdWarning: bool
    version: str
    hasPresence: bool
    hasQrenable: bool
    qrlink: str
    isOpen: bool
    qrMode: int
    schoolEdit: bool



from datetime import date, datetime

import requests

from educabiz.exceptions import LoginFailedError, LoginRequiredError
from educabiz.models.school_qrcodeinfo import SchoolQRCodeInfo

from .models.home import Home


class Client(requests.Session):
    URL = 'https://mobile.educabiz.com'

    def __init__(self, username: str, password: str, login_if_required=False):
        super().__init__()
        self._username = username
        self._password = password
        self._relogin = login_if_required

    def request(self, method, url, *a, login_if_required=True, **b):
        if url[0] == '/':
            url = f'{self.URL}{url}'
        r = super().request(method, url, *a, **b)
        # quick check
        if 'https://mobile.educabiz.com/authenticate' in r.text:
            # sure check
            try:
                data = r.json()
            except Exception:
                return r
            if data.get('formAction') == 'https://mobile.educabiz.com/authenticate':
                if login_if_required and self._relogin:
                    self.login()
                    return super().request(method, url, *a, **b)
                else:
                    raise LoginRequiredError()
        return r

    def login(self):
        r = self.post(
            '/mobile/login', login_if_required=False, data={'username': self._username, 'password': self._password}
        )
        r.raise_for_status()
        r = r.json()
        if r['status'] != 'ok':
            raise LoginFailedError()
        return r

    def home(self):
        r = self.get('/educators/home')
        r.raise_for_status()
        return Home.model_validate(r.json())

    def notifications(self):
        r = self.get('/educators/notifications')
        r.raise_for_status()
        return r.json()

    def child_payments(self, child):
        r = self.get(f'/child/{child}/payments')
        r.raise_for_status()
        return r.json()

    def child_report(self, child, page=0, build_page=False):
        r = self.post(f'/child/{child}/report', data={'page': page, 'buildPage': build_page})
        r.raise_for_status()
        return r.json()

    def child_messages(self, child, page=1):
        r = self.get(f'/child/{child}/messages/income', params={'page': page})
        r.raise_for_status()
        return r.json()

    def child_services(self, child):
        r = self.get(f'/child/{child}/services')
        r.raise_for_status()
        return r.json()

    def child_timetable(self, child):
        r = self.get(f'/child/{child}/timetable')
        r.raise_for_status()
        return r.json()

    def child_gallery(self, child, page=1, build_page=False):
        r = self.get(
            f'/child/{child}/gallery',
            params={
                'page': page,
                'buildPage': build_page,
                'childId': child,
                'serviceId': '',
            },
        )
        r.raise_for_status()
        return r.json()

    def school_qrcodeinfo(self):
        r = self.get(
            '/school/qrcodeinfo',
        )
        r.raise_for_status()
        return SchoolQRCodeInfo.model_validate(r.json())

    def _schoolctrl_save_presence(
        self,
        path: str,
        child_id: str,
        date: date,
        notes='',
        absent=False,
        is_checked=True,
        is_enter=False,
        number_day=1,
    ):
        r = self.post(
            f'/schoolctrl/{path}',
            data={
                'colabId': '',
                'date': date.strftime('%d-%m-%Y'),
                'notes': notes,
                'absent': _bool(absent),
                'isChecked': _bool(is_checked),
                'isEnter': _bool(is_enter),
                'numberDay': number_day,
                'childId': child_id,
            },
        )
        r.raise_for_status()
        return r.json()

    def schoolctrl_save_presence_note(self, child_id: str, date: date, notes=''):
        return self._schoolctrl_save_presence('savepresencesinglenote', child_id, date, notes=notes, absent=True)

    def schoolctrl_save_presence_out(self, child_id: str, date: date):
        return self._schoolctrl_save_presence('savepresenceout', child_id, date)

    def schoolctrl_save_presence_in(self, child_id: str, date: date):
        return self._schoolctrl_save_presence('savepresencein', child_id, date, is_enter=True)

    def child_check_in(self, child_id: str):
        """Check in kid in current day"""
        return self.schoolctrl_save_presence_in(child_id, datetime.now())

    def child_check_out(self, child_id: str):
        """Check out kid in current day"""
        return self.schoolctrl_save_presence_out(child_id, datetime.now())

    def child_absent(self, child_id: str, reason: str):
        """Leave note that kid is absent"""
        return self.schoolctrl_save_presence_note(child_id, datetime.now(), notes=reason)


def _bool(b):
    return 'true' if b else 'false'



import unittest
from unittest import mock

from educabiz import client


class Test(unittest.TestCase):
    @mock.patch('educabiz.client.requests.Session.request')
    def test_client_login(self, request_mock):
        request_mock.return_value.json.return_value = {'status': 'ok'}
        c = client.Client('x', 'y')
        c.login()

    @mock.patch('educabiz.client.requests.Session.request')
    def test_client_login_failed(self, request_mock):
        request_mock.return_value.json.return_value = {'status': 'fail'}
        c = client.Client('x', 'y')
        with self.assertRaises(client.LoginFailedError):
            c.login()



class ClientError(Exception):
    """base class for any error in educabiz.client.Client"""


class LoginFailedError(ClientError):
    """Login invalid"""


class LoginRequiredError(ClientError):
    """Missing or expired session"""



import unittest


class Test(unittest.TestCase):
    def test_presence(self):
        from educabiz.models.school_qrcodeinfo import PresenceItem

        p = PresenceItem.model_validate({'id': '123123', 'notes': 'nice', 'random': 'z', 'hourIn': '--:--'})
        self.assertEqual(p.id, '123123')
        self.assertEqual(p.notes, 'nice')
        self.assertEqual(p.hourOut, '')
        self.assertEqual(p.hourIn, '')
        self.assertEqual(p.model_extra['random'], 'z')

    def test_child_name(self):
        from educabiz.models.school_qrcodeinfo import Child

        c = Child.model_validate({'id': '1', 'presence': [], 'name': 'S&atilde;o T&oacute;'})
        self.assertEqual(c.name, 'São Tó')
